@@ -305,9 +305,8 @@ class TestValidatesDecorator:
 
         schema = BadSchema()
 
-        with pytest.raises(ValueError) as excinfo:
+        with pytest.raises(ValueError, match='"bar" field does not exist.'):
             schema.validate({'foo': 42})
-        assert '"bar" field does not exist.' in str(excinfo)
 
     def test_precedence(self):
         class Schema2(ValidatesSchema):
@@ -336,6 +335,25 @@ class TestValidatesDecorator:
         assert len(errors['bar']) == 1
         assert errors['bar'][0] == 'Must be 2'
 
+    def test_validates_load_from(self):
+        class BadSchema(Schema):
+            foo = fields.String(load_from='foo-name')
+
+            @validates('foo')
+            def validate_string(self, data):
+                raise ValidationError('nope')
+
+        schema = BadSchema()
+        errors = schema.validate({'foo-name': 'data'})
+        assert 'foo-name' in errors
+        assert errors['foo-name'] == ['nope']
+
+        schema = BadSchema()
+        errors = schema.validate([{'foo-name': 'data'}, {'foo-name': 'data2'}], many=True)
+        assert errors == {
+            0: {'foo-name': ['nope']},
+            1: {'foo-name': ['nope']},
+        }
 
 class TestValidatesSchemaDecorator:
 
