@@ -38,9 +38,8 @@ class TestDeserializingNone:
             field = FieldClass(src_str='foo')
         else:
             field = FieldClass()
-        with pytest.raises(ValidationError) as excinfo:
+        with pytest.raises(ValidationError, match='Field may not be null.'):
             field.deserialize(None)
-        assert 'Field may not be null.' in str(excinfo)
 
     def test_allow_none_is_true_if_missing_is_true(self):
         field = fields.Field(missing=None)
@@ -71,6 +70,12 @@ class TestFieldDeserialization:
             field.deserialize(in_val)
         assert excinfo.value.args[0] == 'Not a valid number.'
 
+    def test_float_field_overflow(self):
+        field = fields.Float()
+        with pytest.raises(ValidationError) as excinfo:
+            field.deserialize(2**1024)
+        assert excinfo.value.args[0] == 'Number too large.'
+
     def test_integer_field_deserialization(self):
         field = fields.Integer()
         assert field.deserialize('42') == 42
@@ -305,10 +310,9 @@ class TestFieldDeserialization:
     ])
     def test_invalid_datetime_deserialization(self, in_value):
         field = fields.DateTime()
-        with pytest.raises(ValidationError) as excinfo:
-            field.deserialize(in_value)
         msg = 'Not a valid datetime.'.format(in_value)
-        assert msg in str(excinfo)
+        with pytest.raises(ValidationError, match=msg):
+            field.deserialize(in_value)
 
     def test_datetime_passed_year_is_invalid(self):
         field = fields.DateTime()
@@ -318,23 +322,18 @@ class TestFieldDeserialization:
     def test_custom_date_format_datetime_field_deserialization(self):
 
         dtime = dt.datetime.now()
-        datestring = dtime.strftime('%H:%M:%S %Y-%m-%d')
+        datestring = dtime.strftime('%H:%M:%S.%f %Y-%m-%d')
 
         field = fields.DateTime(format='%d-%m-%Y %H:%M:%S')
         #deserialization should fail when datestring is not of same format
-        with pytest.raises(ValidationError) as excinfo:
+        with pytest.raises(ValidationError, match='Not a valid datetime.'):
             field.deserialize(datestring)
-        msg = 'Not a valid datetime.'.format(datestring)
-        assert msg in str(excinfo)
-        field = fields.DateTime(format='%H:%M:%S %Y-%m-%d')
+        field = fields.DateTime(format='%H:%M:%S.%f %Y-%m-%d')
         assert_datetime_equal(field.deserialize(datestring), dtime)
 
-        field = fields.DateTime()
-        assert msg in str(excinfo)
-
     @pytest.mark.parametrize('fmt', ['rfc', 'rfc822'])
     def test_rfc_datetime_field_deserialization(self, fmt):
-        dtime = dt.datetime.now()
+        dtime = dt.datetime.now().replace(microsecond=0)
         datestring = utils.rfcformat(dtime)
         field = fields.DateTime(format=fmt)
         assert_datetime_equal(field.deserialize(datestring), dtime)
@@ -476,6 +475,12 @@ class TestFieldDeserialization:
             field.deserialize('/foo/bar')
         assert excinfo.value.args[0][0] == 'Not a valid URL.'
 
+    # regression test for https://github.com/marshmallow-code/marshmallow/issues/1400
+    def test_url_field_non_list_validators(self):
+        field = fields.Url(validate=(validate.Length(min=16), ))
+        with pytest.raises(ValidationError, match="Shorter than minimum length"):
+            field.deserialize('https://abc.def')
+
     def test_relative_url_field_deserialization(self):
         field = fields.Url(relative=True)
         assert field.deserialize('/foo/bar') == '/foo/bar'
@@ -495,6 +500,12 @@ class TestFieldDeserialization:
             field.deserialize('invalidemail')
         assert excinfo.value.args[0][0] == 'Not a valid email address.'
 
+    # regression test for https://github.com/marshmallow-code/marshmallow/issues/1400
+    def test_email_field_non_list_validators(self):
+        field = fields.Email(validate=(validate.Length(min=9), ))
+        with pytest.raises(ValidationError, match="Shorter than minimum length"):
+            field.deserialize('a@bc.com')
+
     def test_function_field_deserialization_is_noop_by_default(self):
         field = fields.Function(lambda x: None)
         # Default is noop
@@ -661,9 +672,8 @@ class TestFieldDeserialization:
 
         field = fields.Field(validate=MyValidator())
         assert field.deserialize('valid') == 'valid'
-        with pytest.raises(ValidationError) as excinfo:
+        with pytest.raises(ValidationError, match='Invalid value.'):
             field.deserialize('invalid')
-        assert 'Invalid value.' in str(excinfo)
 
     def test_field_deserialization_with_user_validator_that_raises_error_with_list(self):
         def validator(val):
@@ -704,16 +714,14 @@ class TestFieldDeserialization:
 
         for field in m_colletion_type:
             assert field.deserialize('Valid') == 'Valid'
-            with pytest.raises(ValidationError) as excinfo:
+            with pytest.raises(ValidationError, match='Invalid value.'):
                 field.deserialize('invalid')
-            assert 'Invalid value.' in str(excinfo)
 
     def test_field_deserialization_with_custom_error_message(self):
         field = fields.String(validate=lambda s: s.lower() == 'valid',
                 error_messages={'validator_failed': 'Bad value.'})
-        with pytest.raises(ValidationError) as excinfo:
+        with pytest.raises(ValidationError, match='Bad value.'):
             field.deserialize('invalid')
-        assert 'Bad value.' in str(excinfo)
 
     def test_field_deserialization_with_non_utf8_value(self):
         non_utf8_char = '\xc8'
@@ -1229,10 +1237,9 @@ class TestValidation:
             def get_name(self, val):
                 return val.upper()
         assert MethodSerializer(strict=True).load({'name': 'joe'})
-        with pytest.raises(ValidationError) as excinfo:
+        with pytest.raises(ValidationError, match='Invalid value.'):
             MethodSerializer(strict=True).load({'name': 'joseph'})
 
-        assert 'Invalid value.' in str(excinfo)
 
     # Regression test for https://github.com/marshmallow-code/marshmallow/issues/269
     def test_nested_data_is_stored_when_validation_fails(self):
@@ -1306,9 +1313,8 @@ def test_deserialize_raises_exception_if_strict_is_true_and_input_type_is_incorr
     class MySchema(Schema):
         foo = fields.Field()
         bar = fields.Field()
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='Invalid input type.') as excinfo:
         MySchema(strict=True).load([])
-    assert 'Invalid input type.' in str(excinfo)
     exc = excinfo.value
     assert exc.field_names == ['_schema']
     assert exc.fields == []
