@@ -279,6 +279,16 @@ def test_load_many():
     assert type(result.data[0]) == User
     assert result.data[0].name == 'Mick'
 
+# regression test for https://github.com/marshmallow-code/marshmallow/issues/906
+@pytest.mark.parametrize('val', (False, 1, 1.2, object(), {}, {'1': 2}, 'lol'))
+def test_load_many_invalid_input_type(val):
+    class Sch(Schema):
+        name = fields.Str()
+    with pytest.raises(ValidationError) as e:
+        Sch(strict=True, many=True).load(val)
+    assert e.value.messages == {'_schema': ['Invalid input type.']}
+    assert '_schema' in e.value.field_names
+
 def test_loads_returns_an_unmarshalresult(user):
     s = UserSchema()
     result = s.loads(json.dumps({'name': 'Monty'}))
@@ -539,9 +549,19 @@ def test_prefix(SchemaClass, user):
 def test_fields_must_be_declared_as_instances(user):
     class BadUserSchema(Schema):
         name = fields.String
-    with pytest.raises(TypeError) as excinfo:
+    with pytest.raises(TypeError, match='must be declared as a Field instance'):
         BadUserSchema().dump(user)
-    assert 'must be declared as a Field instance' in str(excinfo)
+
+# regression test
+def test_bind_field_does_not_swallow_typeerror():
+    class MySchema(Schema):
+        name = fields.Str()
+
+        def on_bind_field(self, field_name, field_obj):
+            raise TypeError("boom")
+
+    with pytest.raises(TypeError, match="boom"):
+        MySchema()
 
 @pytest.mark.parametrize('SchemaClass',
     [UserSchema, UserMetaSchema])
@@ -694,6 +714,36 @@ def test_error_raised_if_additional_option_is_not_list():
                 additional = 'email'
 
 
+def test_nested_custom_set_in_exclude_reusing_schema():
+
+    class CustomSet(object):
+        # This custom set is to allow the obj check in BaseSchema.__filter_fields
+        # to pass, since it'll be a valid instance, and this class overrides
+        # getitem method to allow the hasattr check to pass too, which will try
+        # to access the first obj index and will simulate a IndexError throwing.
+        # e.g. SqlAlchemy.Query is a valid use case for this "obj".
+
+        def __getitem__(self, item):
+            return [][item]
+
+    class ChildSchema(Schema):
+        foo = fields.Field(required=True)
+        bar = fields.Field()
+
+        class Meta:
+            only = ('bar', )
+
+    class ParentSchema(Schema):
+        child = fields.Nested(ChildSchema, many=True, exclude=('foo',))
+
+    sch = ParentSchema(strict=True)
+    obj = dict(child=CustomSet())
+    sch.dumps(obj)
+    data = dict(child=[{'bar': 1}])
+    result = sch.load(data, partial=True)
+    assert not result.errors
+
+
 def test_nested_only():
     class ChildSchema(Schema):
         foo = fields.Field()
@@ -887,6 +937,34 @@ def test_nested_exclude_and_only_inheritance():
     assert 'fuu' not in child
 
 
+# https://github.com/marshmallow-code/marshmallow/issues/1160
+def test_nested_instance_many():
+    class BookSchema(Schema):
+        id = fields.Int()
+        title = fields.String()
+
+        class Meta:
+            strict = True
+
+
+    class UserSchema(Schema):
+        id = fields.Int()
+        name = fields.String()
+        books = fields.Nested(BookSchema(many=True))
+
+        class Meta:
+            strict = True
+
+    books = [{'id': 1, 'title': 'First book'}, {'id': 2, 'title': 'Second book'}]
+    user = {'id': 1, 'name': 'Peter', 'books': books}
+
+    user_dump = UserSchema().dump(user).data
+    assert user_dump['books'] == books
+
+    user_load = UserSchema().load(user_dump).data
+    assert user_load == user
+
+
 def test_meta_nested_exclude():
     class ChildSchema(Schema):
         foo = fields.Field()
@@ -898,8 +976,22 @@ def test_meta_nested_exclude():
         blubb = fields.Nested(ChildSchema)
         class Meta:
             exclude = ('blubb.foo',)
-    sch = ParentSchema()
+
     data = dict(bla=1, bli=2, blubb=dict(foo=42, bar=24, baz=242))
+
+    sch = ParentSchema()
+    result = sch.dump(data)
+    assert 'bla' in result.data
+    assert 'blubb' in result.data
+    assert 'bli' in result.data
+    child = result.data['blubb']
+    assert 'foo' not in child
+    assert 'bar' in child
+    assert 'baz' in child
+
+    # Test fields with dot notations in Meta.exclude on multiple instantiations
+    # Regression test for https://github.com/marshmallow-code/marshmallow/issues/1212
+    sch = ParentSchema()
     result = sch.dump(data)
     assert 'bla' in result.data
     assert 'blubb' in result.data
@@ -1574,6 +1666,21 @@ class TestNestedSchema:
         for i, name in enumerate(data['collaborators']):
             assert name == blog.collaborators[i].name
 
+    # Regression test for https://github.com/marshmallow-code/marshmallow/issues/800
+    def test_flat_nested_with_dump_to(self, blog):
+        class UserSchema(Schema):
+            name = fields.String(dump_to='username')
+            age = fields.Int()
+        class FlatBlogSchema(Schema):
+            name = fields.String()
+            user = fields.Nested(UserSchema, only='name')
+            collaborators = fields.Nested(UserSchema, only='name', many=True)
+        s = FlatBlogSchema()
+        data, _ = s.dump(blog)
+        assert data['user'] == blog.user.name
+        for i, name in enumerate(data['collaborators']):
+            assert name == blog.collaborators[i].name
+
     # regression test for https://github.com/marshmallow-code/marshmallow/issues/64
     def test_nested_many_with_missing_attribute(self, user):
         class SimpleBlogSchema(Schema):
@@ -1699,11 +1806,10 @@ class TestNestedSchema:
         assert "collaborators" not in errors
 
     def test_nested_strict(self):
-        with pytest.raises(ValidationError) as excinfo:
+        with pytest.raises(ValidationError, match='email'):
             _, errors = BlogSchema(strict=True).load(
                 {'title': "Monty's blog", 'user': {'name': 'Monty', 'email': 'foo'}}
             )
-        assert 'email' in str(excinfo)
 
     def test_nested_dump_errors(self, blog):
         blog.user.email = "foo"
@@ -1716,9 +1822,8 @@ class TestNestedSchema:
 
     def test_nested_dump_strict(self, blog):
         blog.user.email = "foo"
-        with pytest.raises(ValidationError) as excinfo:
+        with pytest.raises(ValidationError, match='email'):
             _, errors = BlogSchema(strict=True).dump(blog)
-        assert 'email' in str(excinfo)
 
     def test_nested_method_field(self, blog):
         data = BlogSchema().dump(blog)[0]
@@ -2058,10 +2163,9 @@ class TestContext:
         serializer = UserFunctionContextSchema(strict=True)
         # no context
         serializer.context = None
-        with pytest.raises(ValidationError) as excinfo:
-            serializer.dump(owner)
         msg = 'No context available for Function field {0!r}'.format('is_collab')
-        assert msg in str(excinfo)
+        with pytest.raises(ValidationError, match=msg) as excinfo:
+            serializer.dump(owner)
 
     def test_fields_context(self):
         class CSchema(Schema):
@@ -2087,6 +2191,27 @@ class TestContext:
         result = ser.dump(obj)
         assert result.data['inner']['likes_bikes'] is True
 
+    # Regression test for https://github.com/marshmallow-code/marshmallow/issues/820
+    def test_nested_list_fields_inherit_context(self):
+        class InnerSchema(Schema):
+            foo = fields.Field()
+
+            @validates('foo')
+            def validate_foo(self, value):
+                if 'foo_context' not in self.context:
+                    raise ValidationError('Missing context')
+
+        class OuterSchema(Schema):
+            bars = fields.List(fields.Nested(InnerSchema()))
+
+        inner = InnerSchema(strict=True)
+        inner.context['foo_context'] = 'foo'
+        assert inner.load({'foo': 42})
+
+        outer = OuterSchema(strict=True)
+        outer.context['foo_context'] = 'foo'
+        assert outer.load({'bars': [{'foo': 42}]})
+
 
 def test_serializer_can_specify_nested_object_as_attribute(blog):
     class BlogUsernameSchema(Schema):
