@@ -23,6 +23,8 @@ from marshmallow import validate, ValidationError
     'http://123.45.67.8/',
     'http://2001:db8::ff00:42:8329',
     'http://www.example.com:8000/foo',
+    'http://user@example.com',
+    'http://user:pass@example.com',
 ])
 def test_url_absolute_valid(valid_url):
     validator = validate.URL(relative=False)
@@ -39,6 +41,7 @@ def test_url_absolute_valid(valid_url):
     'http:/example.org',
     'foo://example.org',
     '../icons/logo.gif',
+    'https://example.org\n',
     'abc',
     '..',
     '/',
@@ -68,7 +71,8 @@ def test_url_relative_valid(valid_url):
     'http//example.org',
     'suppliers.html',
     '../icons/logo.gif',
-    '\icons/logo.gif',
+    'icons/logo.gif',
+    'http://example.org\n',
     '../.../g',
     '...',
     '\\',
@@ -96,6 +100,7 @@ def test_url_dont_require_tld_valid(valid_url):
 
 @pytest.mark.parametrize('invalid_url', [
     'http//example',
+    'http://example\n',
     'http://.example.org',
     'http:///foo/bar',
     'http:// /foo/bar',
@@ -129,9 +134,8 @@ def test_url_relative_and_custom_schemes():
 
 def test_url_custom_message():
     validator = validate.URL(error="{input} ain't an URL")
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match="invalid ain't an URL"):
         validator('invalid')
-    assert "invalid ain't an URL" in str(excinfo)
 
 def test_url_repr():
     assert (
@@ -164,6 +168,8 @@ def test_email_valid(valid_email):
     assert validator(valid_email) == valid_email
 
 @pytest.mark.parametrize('invalid_email', [
+    'niceandsimple\n@example.com',
+    'NiCeAnDsImPlE@eXaMpLe.CoM\n',
     'a"b(c)d,e:f;g<h>i[j\\k]l@example.com',
     'just"not"right@example.com',
     'this is"not\allowed@example.com',
@@ -185,9 +191,8 @@ def test_email_invalid(invalid_email):
 
 def test_email_custom_message():
     validator = validate.Email(error='{input} is not an email addy.')
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='invalid is not an email addy.'):
         validator('invalid')
-    assert 'invalid is not an email addy.' in str(excinfo)
 
 def test_email_repr():
     assert (
@@ -226,19 +231,16 @@ def test_range_max():
 
 def test_range_custom_message():
     v = validate.Range(2, 3, error='{input} is not between {min} and {max}')
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='1 is not between 2 and 3'):
         v(1)
-    assert '1 is not between 2 and 3' in str(excinfo)
 
     v = validate.Range(2, None, error='{input} is less than {min}')
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='1 is less than 2'):
         v(1)
-    assert '1 is less than 2' in str(excinfo)
 
     v = validate.Range(None, 3, error='{input} is greater than {max}')
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='4 is greater than 3'):
         v(4)
-    assert '4 is greater than 3' in str(excinfo)
 
 def test_range_repr():
     assert (
@@ -310,24 +312,20 @@ def test_length_equal():
 
 def test_length_custom_message():
     v = validate.Length(5, 6, error='{input} is not between {min} and {max}')
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='foo is not between 5 and 6'):
         v('foo')
-    assert 'foo is not between 5 and 6' in str(excinfo)
 
     v = validate.Length(5, None, error='{input} is shorter than {min}')
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='foo is shorter than 5'):
         v('foo')
-    assert 'foo is shorter than 5' in str(excinfo)
 
     v = validate.Length(None, 2, error='{input} is longer than {max}')
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='foo is longer than 2'):
         v('foo')
-    assert 'foo is longer than 2' in str(excinfo)
 
     v = validate.Length(None, None, equal=4, error='{input} does not have {equal}')
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='foo does not have 4'):
         v('foo')
-    assert 'foo does not have 4' in str(excinfo)
 
 def test_length_repr():
     assert (
@@ -360,9 +358,8 @@ def test_equal():
 
 def test_equal_custom_message():
     v = validate.Equal('a', error='{input} is not equal to {other}.')
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='b is not equal to a.'):
         v('b')
-    assert 'b is not equal to a.' in str(excinfo)
 
 def test_equal_repr():
     assert (
@@ -413,9 +410,8 @@ def test_regexp_compile():
 def test_regexp_custom_message():
     rex = r'[0-9]+'
     v = validate.Regexp(rex, error='{input} does not match {regex}')
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='a does not match'):
         v('a')
-    assert 'a does not match [0-9]+' in str(excinfo)
 
 def test_regexp_repr():
     assert (
@@ -455,9 +451,8 @@ def test_predicate():
     assert validate.Predicate('_identity', arg=1)(d) == d
     assert validate.Predicate('_identity', arg='abc')(d) == d
 
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='Invalid input.'):
         validate.Predicate('_false')(d)
-    assert 'Invalid input.' in str(excinfo)
     with pytest.raises(ValidationError):
         validate.Predicate('_empty')(d)
     with pytest.raises(ValidationError):
@@ -475,9 +470,8 @@ def test_predicate_custom_message():
         def __str__(self):
             return 'Dummy'
     d = Dummy()
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='Dummy._false is invalid!'):
         validate.Predicate('_false', error='{input}.{method} is invalid!')(d)
-    assert 'Dummy._false is invalid!' in str(excinfo)
 
 def test_predicate_repr():
     assert (
@@ -500,9 +494,8 @@ def test_noneof():
     assert validate.NoneOf([])([]) == []
     assert validate.NoneOf([1, 2, 3])(None) is None
 
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='Invalid input.'):
         validate.NoneOf([1, 2, 3])(3)
-    assert 'Invalid input.' in str(excinfo)
     with pytest.raises(ValidationError):
         validate.NoneOf('abc')('c')
     with pytest.raises(ValidationError):
@@ -511,17 +504,15 @@ def test_noneof():
         validate.NoneOf('')('')
 
 def test_noneof_custom_message():
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='<not valid>'):
         validate.NoneOf([1, 2], error='<not valid>')(1)
-    assert '<not valid>' in str(excinfo)
 
     none_of = validate.NoneOf(
         [1, 2],
         error='{input} cannot be one of {values}'
     )
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='1 cannot be one of 1, 2'):
         none_of(1)
-    assert '1 cannot be one of 1, 2' in str(excinfo)
 
 def test_noneof_repr():
     assert (
@@ -543,9 +534,8 @@ def test_oneof():
     assert validate.OneOf(dict(a=0, b=1))('a') == 'a'
     assert validate.OneOf((1, 2, None))(None) is None
 
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='Not a valid choice.'):
         validate.OneOf([1, 2, 3])(4)
-    assert 'Not a valid choice.' in str(excinfo)
     with pytest.raises(ValidationError):
         validate.OneOf('abc')('d')
     with pytest.raises(ValidationError):
@@ -592,7 +582,7 @@ def test_oneof_text():
 def test_oneof_custom_message():
     oneof = validate.OneOf([1, 2, 3], error='{input} is not one of {choices}')
     expected = '4 is not one of 1, 2, 3'
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError):
         oneof(4)
     assert expected in str(expected)
 
@@ -601,7 +591,7 @@ def test_oneof_custom_message():
         error='{input} is not one of {labels}'
     )
     expected = '4 is not one of one, two, three'
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError):
         oneof(4)
     assert expected in str(expected)
 
@@ -687,9 +677,8 @@ def test_contains_only_in_string():
         validate.ContainsOnly('')('a')
 
 def test_contains_only_invalid():
-    with pytest.raises(ValidationError) as excinfo:
+    with pytest.raises(ValidationError, match='One or more of the choices you made was not acceptable.'):
         validate.ContainsOnly([1, 2, 3])([1, 1])
-    assert 'One or more of the choices you made was not acceptable.' in str(excinfo)
     with pytest.raises(ValidationError):
         validate.ContainsOnly([1, 1, 2])([2, 2])
     with pytest.raises(ValidationError):
